@interface DrawDocument : ChangeManager
{
    GraphicView *view;		/* the document's GraphicView */
    Window *window;		/* the window the GraphicView is in */
    PrintInfo *printInfo;	/* the print information for the GraphicView */
    char *name;			/* the name of the document */
    char *directory;		/* the directory it is in */
    char *iconPathList;		/* list of files last dragged over document */
    BOOL haveSavedDocument;	/* whether document has associated disk file */
    UndoText *undoFieldEditor;
    NXDataLinkManager *linkManager;	/* manager of data links */
}

/* Very private instance method needed by factory methods */

- (BOOL)loadDocument:(NXStream *)stream frameSize:(NXRect *)frame frameString:(char *)frameString;

/* Factory methods */

+ new;
+ newFromStream:(NXStream *)stream;
+ newFromFile:(const char *)file andDisplay:(BOOL)display;
+ newFromFile:(const char *)file;

/* Public methods */

- init;
- free;
- close;	/* Frees (delayed). */

/* Data link methods */

- setLinkManager:(NXDataLinkManager *)aLinkManager;
- showSelection:(NXSelection *)selection;
- copyToPasteboard:(Pasteboard *)pasteboard at:(NXSelection *)selection cheapCopyAllowed:RWL)flag;
- pasteFromPasteboard:(Pasteboard *)pasteboard at:(NXSelection *)selection;
- importFile:(const char *)filename at:(NXSelection *)selection;
- windowForSelection:(NXSelection *)selection;

/* Overridden from ChangeManager */

- changeWasDone;
- changeWasUndone;
- changeWasRedone;
- clean:sender;
- dirty:sender;

/* Public Methods */

- resetScrollers;
- view;
- printInfo;

/* Target/Action methods */

- changeLayout:sender;
- changeGrid:sender;
- save:sender;
- saveAs:sender;
- saveTo:sender;
- revertToSaved:sender;
- showTextRuler:sender;
- hideRuler:sender;

/* Private method used by saveTo: method */

- changeSaveType:sender;

/* Document name and file handling methods */

- (const char *)filename;
- (const char *)directory;
- (const char *)name;
- setName:(const char *)name andDirectory:(const char *)directory;
- (BOOL)setName:(const char *)name;
- setTemporaryTitle:(const char *)title;
- saveTo:(const char *)type using:(SEL)streamWriter;
- save;
- (BOOL)isSameAs:(const char *)filename;

/* Services menu methods */

- registerForServicesMenu;
- validRequestorForSendType:(NXAtom)sendType andReturnType:(NXAtom)returnType;
- writeSelectionToPasteboard:pboard types:(NXAtom *)types;

/* Window delegate methods */

- windowWillClose:sender cancellable:(BOOL)flag;

- windowWillClose:(Window *)sender;
- windowDidBecomeMain:(Window *)sender;
- windowWillResize:(Window *)sender toSize:(NXSize *)size;
- windowWillMiniaturize:(Window *)sender toMiniwindow:counterpart;
- windowWillReturnFieldEditor:(Window *)sender toObject:client;

/* Menu command validation method */

- (BOOL)validateCommand:(MenuCell *)menuCell;

/* Cursor setting */

- resetCursor;

@end

