@interface Group : Graphic
{
    NXImage *cache;	/* an NXImage used to cache the group */
    List *components;	/* the Graphics in the group */
    NXRect lastRect;	/* the last rectangle the group was drawn in */
    BOOL dontCache;	/* whether we can cache this group */
    BOOL hasTextGraphic;/* whether a TextGraphic is included in the group */
}

/* Creation methods */

- initList:(List *)list;
- free;

/* Public methods */

- transferSubGraphicsTo:(List *)list at:(int)position;
- (List *)subGraphics;

/* Group must override all the setting routines to forward to components */

- makeGraphicsPerform:(SEL)aSelector with:(const void *)anArgument;

- changeFont:sender;
- (Font *)font;
- setLineWidth:(const float *)value;
- setGray:(const float *)value;
- setFill:(int)mode;
- setFillColor:(NXColor *)aColor;
- setLineColor:(NXColor *)aColor;
- setLineCap:(int)capValue;
- setLineArrow:(int)arrowValue;
- setLineJoin:(int)joinValue;

/* Link methods */

- reviveLink:(NXDataLinkManager *)linkManager;
- (Graphic *)graphicLinkedBy:(NXDataLink *)aLink;

- resetIdentifier;
- writeIdentifierTo:(char *)buffer;
- (int)graphicCount;
- (Graphic *)graphicIdentifiedBy:(int)anIdentifier;

/* Notification from GraphicView */

- wasRemovedFrom:(GraphicView *)sender;
- wasAddedTo:(GraphicView *)sender;

/* Methods overridden from superclass */

- (Graphic *)colorAcceptorAt:(const NXPoint *)point;
- (BOOL)hasTextGraphic;
- setCacheable:(BOOL)flag;
- (BOOL)isCacheable;
- draw;
- (BOOL)hit:(const NXPoint *)point;

/* Compatibility methods */

- replaceWithImage;
RWArchiving methods */

- write:(NXTypedStream *)stream;
- read:(NXTypedStream *)stream;

@end
